import os
from typing import Any

import easymaker
from easymaker.api.request_body import TrainingCreateBody
from easymaker.common import utils
from easymaker.common.base_model.training_base_model import TrainingBaseModel


class Training(TrainingBaseModel):

    def _fetch(self):
        response = easymaker.easymaker_config.api_sender.get_training_by_id(self.training_id)
        super().__init__(**response)

    def run(
        self,
        training_name: str,
        experiment_id: str | None = None,
        description: str | None = None,
        image_name: str | None = None,
        instance_name: str | None = None,
        distributed_node_count: int | None = 1,
        data_storage_size: int | None = None,
        source_dir_uri: str | None = None,
        entry_point: str | None = None,
        algorithm_name: str | None = None,
        hyperparameter_list: list[Any] | None = None,  # [{"name": "","type": easymaker.HYPERPARAMETER_TYPE_CODE,"feasibleSpace": {"min": "","max": "","list": "","step": "",}}, ]
        dataset_list: list[Any] | None = None,
        check_point_input_uri: str | None = None,
        check_point_upload_uri: str | None = None,
        model_upload_uri: str | None = None,
        timeout_hours: int | None = 720,
        tag_list: list[Any] | None = None,
        use_log: bool | None = False,
        wait: bool | None = True,
        use_torchrun: bool | None = False,
        nproc_per_node: int | None = 0,
    ):
        """
        Returns:
            training_id
        """
        # run training
        instance_list = easymaker.easymaker_config.api_sender.get_instance_list()
        image_list = easymaker.easymaker_config.api_sender.get_image_list()
        algorithm_list = easymaker.easymaker_config.api_sender.get_algorithm_list()
        if not experiment_id:
            experiment_id = os.environ.get("EM_EXPERIMENT_ID")
        response = easymaker.easymaker_config.api_sender.run_training(
            TrainingCreateBody(
                training_name=training_name,
                description=description,
                experiment_id=experiment_id,
                image_id=utils.from_name_to_id(image_list, image_name, "image"),
                flavor_id=utils.from_name_to_id(instance_list, instance_name, "instance"),
                instance_count=distributed_node_count,
                data_storage_size=data_storage_size,
                source_dir_uri=source_dir_uri,
                entry_point=entry_point,
                algorithm_id=utils.from_name_to_id(algorithm_list, algorithm_name, "algorithm") if algorithm_name else None,
                hyperparameter_list=hyperparameter_list,
                dataset_list=dataset_list,
                check_point_input_uri=check_point_input_uri,
                check_point_upload_uri=check_point_upload_uri,
                model_upload_uri=model_upload_uri,
                training_type_code="NORMAL",
                timeout_minutes=timeout_hours * 60,
                tag_list=tag_list,
                use_log=use_log,
                use_torchrun=use_torchrun,
                nproc_per_node=nproc_per_node,
            )
        )
        super().__init__(**response)
        print(f"[AI EasyMaker] Training create request complete. training_id: {self.training_id}")
        if wait:
            self.wait()

        return self

    def delete(self):
        if self.training_id:
            easymaker.easymaker_config.api_sender.delete_training_by_id(self.training_id)
            super().__init__()
            print(f"[AI EasyMaker] Training delete request complete. Training ID : {self.training_id}")
        else:
            print("[AI EasyMaker] Failed to delete training. The training_id is empty.")


def delete(training_id: str):
    if training_id:
        easymaker.easymaker_config.api_sender.delete_training_by_id(training_id)
        print(f"[AI EasyMaker] Training delete request complete. Training ID : {training_id}")
    else:
        print("[AI EasyMaker] Failed to delete training. The training_id is empty.")
